@@ -53,23 +53,25 @@ extern "C" {
  * \defgroup DSP_Defines DSP API defines
 */
 /*@{*/
-
-#ifndef dsp_t
+#define DSP_MAX_STARS 200
 #define dsp_t double
-#endif
+#define dsp_t_max pow(2, sizeof(dsp_t)*4)
+#define dsp_t_min -dsp_t_max
+
+#define DSP_MAX_THREADS 4
 
 ///if min() is not present you can use this one
 #ifndef Min
 #define Min(a,b) \
    ({ __typeof__ (a) _a = (a); \
-       __typeof__ (b) _b = (b); \
+       __typeof__ (a) _b = (b); \
      _a < _b ? _a : _b; })
 #endif
 ///if max() is not present you can use this one
 #ifndef Max
 #define Max(a,b) \
    ({ __typeof__ (a) _a = (a); \
-       __typeof__ (b) _b = (b); \
+       __typeof__ (a) _b = (b); \
      _a > _b ? _a : _b; })
 #endif
 ///Logarithm of a with arbitrary base b
@@ -90,11 +92,39 @@ extern "C" {
 typedef struct dsp_point_t
 {
 /// Center of the point
-    int* center;
+    double* location;
 /// Dimensions limit of the point
     int dims;
 } dsp_point;
 
+/**
+* \brief Indicates an offset
+*/
+typedef struct dsp_offset_t
+{
+/// Center of the point
+    double* offset;
+/// Dimensions limit of the point
+    int dims;
+} dsp_offset;
+
+/**
+* \brief Alignment informations needed
+*/
+typedef struct dsp_align_info_t
+{
+    /// Traslation offset
+    double* offset;
+    /// Center of rotation coordinates
+    double* center;
+    /// Rotational offset
+    double* radians;
+    /// Scaling factor
+    double factor;
+    /// Dimensions limit
+    int dims;
+} dsp_align_info;
+
 /**
 * \brief Complex number, used in Fourier Transform functions
 * \sa dsp_fourier_dft
@@ -126,8 +156,8 @@ typedef struct dsp_star_t
 {
 /// The center of the star
     dsp_point center;
-/// The radius of the star
-    int radius;
+/// The diameter of the star
+    double diameter;
 } dsp_star;
 
 /**
@@ -167,7 +197,17 @@ typedef struct dsp_stream_t
 /// Time at the beginning of the stream
     struct timespec starttimeutc;
 /// Wavelength observed, used as reference with signal generators or filters
-    double lambda;
+    double wavelength;
+/// Focal ratio
+    double focal_ratio;
+/// Diameter
+    double diameter;
+/// SNR
+    double SNR;
+/// Red pixel (Bayer)
+    int red;
+/// Sensor size
+    double *pixel_sizes;
 /// Sample rate of the buffers
     double samplerate;
 /// Thread type for future usage
@@ -177,9 +217,13 @@ typedef struct dsp_stream_t
 /// Regions of interest for each dimension
     dsp_region *ROI;
 /// Stars or objects identified into the buffers - TODO
-    dsp_star **stars;
-/// Stars or objects quantity - TODO
-    int star_count;
+    dsp_star *stars;
+/// Stars or objects quantity
+    int stars_count;
+/// Align/scale/rotation settings
+    dsp_align_info align_info;
+/// Frame number (if part of a series)
+    int frame_number;
 } dsp_stream, *dsp_stream_p;
 
 /*@}*/
@@ -190,12 +234,28 @@ typedef struct dsp_stream_t
 
 /**
 * \brief Perform a discrete Fourier Transform of a dsp_stream
-* \param stream the input stream.
-* \return the output stream if successfull elaboration. NULL if an
-* error is encountered.
+* \param stream the inout stream.
 */
 DLL_EXPORT dsp_complex* dsp_fourier_dft(dsp_stream_p stream);
 
+/**
+* \brief Perform an inverse discrete Fourier Transform of a dsp_stream
+* \param stream the inout stream.
+*/
+DLL_EXPORT dsp_t* dsp_fourier_idft(dsp_stream_p stream);
+
+/**
+* \brief Perform a fast Fourier Transform of a dsp_stream
+* \param stream the inout stream.
+*/
+DLL_EXPORT void dsp_fourier_fft(dsp_stream_p stream);
+
+/**
+* \brief Perform an inverse fast Fourier Transform of a dsp_stream
+* \param stream the inout stream.
+*/
+DLL_EXPORT void dsp_fourier_ifft(dsp_stream_p stream);
+
 /**
 * \brief Obtain a complex number's magnitude
 * \param n the input complex.
@@ -238,6 +298,18 @@ DLL_EXPORT void dsp_fourier_dft_magnitude(dsp_stream_p stream);
 */
 DLL_EXPORT void dsp_fourier_dft_phase(dsp_stream_p stream);
 
+/**
+* \brief Perform an inverse discrete Fourier Transform of a dsp_stream and obtain the complex magnitudes
+* \param stream the input stream.
+*/
+DLL_EXPORT void dsp_fourier_idft_magnitude(dsp_stream_p stream);
+
+/**
+* \brief Perform an inverse discrete Fourier Transform of a dsp_stream and obtain the complex phases
+* \param stream the input stream.
+*/
+DLL_EXPORT void dsp_fourier_idft_phase(dsp_stream_p stream);
+
 /*@}*/
 /**
  * \defgroup dsp_Filters DSP API Linear buffer filtering functions
@@ -293,7 +365,7 @@ DLL_EXPORT void dsp_filter_bandreject(dsp_stream_p stream, double samplingfreque
 * \param stream1 the first input stream.
 * \param stream2 the second input stream.
 */
-DLL_EXPORT void dsp_convolution_convolution(dsp_stream_p stream1, dsp_stream_p stream2);
+DLL_EXPORT dsp_stream_p dsp_convolution_convolution(dsp_stream_p stream1, dsp_stream_p stream2);
 
 /*@}*/
 /**
@@ -343,7 +415,7 @@ DLL_EXPORT void dsp_convolution_convolution(dsp_stream_p stream1, dsp_stream_p s
 ({\
     int i;\
     __typeof__(buf[0]) min = dsp_stats_min(buf, len);\
-    (__typeof__(buf[0]))(min - dsp_stats_max(buf, len)) / 2.0 + min);\
+    (__typeof__(buf[0]))(min - dsp_stats_max(buf, len)) / 2.0 + min;\
 })
 
 /**
@@ -413,6 +485,24 @@ DLL_EXPORT void dsp_convolution_convolution(dsp_stream_p stream1, dsp_stream_p s
     count;\
     })
 
+/**
+* \brief Counts value occurrences into stream
+* \param buf the input buffer
+* \param len the length in elements of the buffer.
+* \param val the value to count.
+* \return the count of the value of the stream.
+*/
+#define dsp_stats_range_count(buf, len, lo, hi) \
+({\
+    int x;\
+    int count = 0;\
+    for(x = 0; x < len; x++) {\
+        if(buf[x] < hi && buf[x] >= lo)\
+            count ++;\
+    }\
+    count;\
+    })
+
 /**
 * \brief Compare two streams
 * \param in1 the first input buffer
@@ -436,7 +526,7 @@ DLL_EXPORT void dsp_convolution_convolution(dsp_stream_p stream1, dsp_stream_p s
 * \return the output stream if successfull elaboration. NULL if an
 * error is encountered.
 */
-DLL_EXPORT dsp_t* dsp_stats_histogram(dsp_stream_p stream, int size);
+DLL_EXPORT double* dsp_stats_histogram(dsp_stream_p stream, int size);
 
 /*@}*/
 /**
@@ -467,15 +557,30 @@ DLL_EXPORT void dsp_buffer_removemean(dsp_stream_p stream);
 #define dsp_buffer_stretch(buf, len, _mn, _mx)\
 ({\
     int k;\
-    __typeof__(buf[0]) mn = dsp_stats_min(buf, len);\
-    __typeof__(buf[0]) mx = dsp_stats_max(buf, len);\
+    __typeof__(buf[0]) __mn = dsp_stats_min(buf, len);\
+    __typeof__(buf[0]) __mx = dsp_stats_max(buf, len);\
     double oratio = (_mx - _mn);\
-    double iratio = (mx - mn);\
-    if(iratio == 0.0) iratio = 1;\
+    double iratio = (__mx - __mn);\
+    if(iratio == 0) iratio = 1;\
     for(k = 0; k < len; k++) {\
-        buf[k] -= mn;\
+        buf[k] -= __mn;\
         buf[k] = (__typeof__(buf[0]))((double)buf[k] * oratio / iratio);\
-        buf[k] += (__typeof__(buf[0]))_mn;\
+        buf[k] += _mn;\
+    }\
+})
+
+/**
+* \brief Place the given value on each element of the buffer
+* \param buf the input buffer
+* \param len the length in elements of the buffer.
+* \param val the desired value.
+*/
+
+#define dsp_buffer_set(buf, len, _val)\
+({\
+    int k;\
+    for(k = 0; k < len; k++) {\
+        buf[k] = (__typeof__(buf[0]))(_val);\
     }\
 })
 
@@ -486,14 +591,30 @@ DLL_EXPORT void dsp_buffer_removemean(dsp_stream_p stream);
 * \param min the clamping minimum value.
 * \param max the clamping maximum value.
 */
-#define dsp_buffer_normalize(buf, len, min, max)\
+#define dsp_buffer_normalize(buf, len, mn, mx)\
 ({\
-        int k;\
+    int k;\
     for(k = 0; k < len; k++) {\
-        buf[k] = (buf[k] < min ? min : (buf[k] > max ? max : buf[k]));\
-        }\
+        buf[k] = Max(mn, Min(mx, buf[k]));\
+    }\
 })
 
+/**
+* \brief Subtract elements of one stream from another's
+* \param stream the stream on which execute
+* \param in the buffer operand.
+* \param len the length of the buffer
+*/
+DLL_EXPORT void dsp_buffer_max(dsp_stream_p stream, dsp_t* in, int len);
+
+/**
+* \brief Sum elements of one stream to another's
+* \param stream the stream on which execute
+* \param in the buffer operand.
+* \param len the length of the buffer
+*/
+DLL_EXPORT void dsp_buffer_min(dsp_stream_p stream, dsp_t* in, int len);
+
 /**
 * \brief Subtract elements of one stream from another's
 * \param stream the stream on which execute
@@ -516,7 +637,7 @@ DLL_EXPORT void dsp_buffer_sum(dsp_stream_p stream, dsp_t* in, int len);
 * \param in the buffer operand.
 * \param len the length of the buffer
 */
-DLL_EXPORT void dsp_buffer_div(dsp_stream_p stream, double* in, int len);
+DLL_EXPORT void dsp_buffer_div(dsp_stream_p stream, dsp_t* in, int len);
 
 /**
 * \brief Multiply elements of one stream to another's
@@ -524,7 +645,7 @@ DLL_EXPORT void dsp_buffer_div(dsp_stream_p stream, double* in, int len);
 * \param in the buffer operand.
 * \param len the length of the buffer
 */
-DLL_EXPORT void dsp_buffer_mul(dsp_stream_p stream, double* in, int len);
+DLL_EXPORT void dsp_buffer_mul(dsp_stream_p stream, dsp_t* in, int len);
 
 /**
 * \brief Expose elements of one stream to another's
@@ -532,7 +653,7 @@ DLL_EXPORT void dsp_buffer_mul(dsp_stream_p stream, double* in, int len);
 * \param in the buffer operand.
 * \param len the length of the buffer
 */
-DLL_EXPORT void dsp_buffer_pow(dsp_stream_p stream, double* in, int len);
+DLL_EXPORT void dsp_buffer_pow(dsp_stream_p stream, dsp_t* in, int len);
 
 /**
 * \brief Logarithm elements of one stream using another's as base
@@ -540,28 +661,28 @@ DLL_EXPORT void dsp_buffer_pow(dsp_stream_p stream, double* in, int len);
 * \param in the buffer operand.
 * \param len the length of the buffer
 */
-DLL_EXPORT void dsp_buffer_log(dsp_stream_p stream, double* in, int len);
+DLL_EXPORT void dsp_buffer_log(dsp_stream_p stream, dsp_t* in, int len);
 
 /**
 * \brief Subtract a value from elements of the input stream
 * \param stream the stream on which execute
 * \param val the value to be subtracted.
 */
-DLL_EXPORT void dsp_buffer_sub1(dsp_stream_p stream, dsp_t val);
+DLL_EXPORT void dsp_buffer_sub1(dsp_stream_p stream, double val);
 
 /**
 * \brief Subtract each element of the input stream a value
 * \param stream the stream on which execute
 * \param val the value to be subtracted.
 */
-DLL_EXPORT void dsp_buffer_1sub(dsp_stream_p stream, dsp_t val);
+DLL_EXPORT void dsp_buffer_1sub(dsp_stream_p stream, double val);
 
 /**
 * \brief Sum elements of the input stream to a value
 * \param stream the stream on which execute
 * \param val the value used for this operation.
 */
-DLL_EXPORT void dsp_buffer_sum1(dsp_stream_p stream, dsp_t val);
+DLL_EXPORT void dsp_buffer_sum1(dsp_stream_p stream, double val);
 
 /**
 * \brief Divide elements of the input stream to a value
@@ -606,12 +727,6 @@ DLL_EXPORT void dsp_buffer_log1(dsp_stream_p stream, double val);
 */
 DLL_EXPORT void dsp_buffer_median(dsp_stream_p stream, int size, int median);
 
-/**
-* \brief Put zero on each element of the array
-* \param stream the stream on which execute
-*/
-DLL_EXPORT void dsp_buffer_clear(dsp_stream_p stream);
-
 /**
 * \brief Deviate forward the first input stream using the second stream as indexing reference
 * \param stream the stream on which execute
@@ -619,7 +734,7 @@ DLL_EXPORT void dsp_buffer_clear(dsp_stream_p stream);
 * \param mindeviation the deviation at 0.
 * \param maxdeviation the deviation at 1.
 */
-DLL_EXPORT void dsp_buffer_deviate(dsp_stream_p stream, double* deviation, double mindeviation, double maxdeviation);
+DLL_EXPORT void dsp_buffer_deviate(dsp_stream_p stream, dsp_t* deviation, dsp_t mindeviation, dsp_t maxdeviation);
 
 /**
 * \brief Reverse the order of the buffer elements
@@ -635,14 +750,35 @@ DLL_EXPORT void dsp_buffer_deviate(dsp_stream_p stream, double* deviation, doubl
         while(i >= 0) \
         { \
           _x = buf[j]; \
-          buf[i] = buf[j]; \
-          buf[j] = _x; \
+          buf[j] = buf[i]; \
+          buf[i] = _x; \
           i--; \
           j++; \
         } \
     })
 #endif
 
+#ifndef dsp_buffer_swap
+#define dsp_buffer_swap(in, len) \
+    ({ \
+        int k; \
+        switch(sizeof(((__typeof__ (in[0])*)in)[0])) { \
+        case 2: \
+            for(k = 0; k < len; k++) \
+                ((__typeof__ (in[0])*)in)[k] = __bswap_16(((__typeof__ (in[0])*)in)[k]); \
+            break; \
+        case 3: \
+            for(k = 0; k < len; k++) \
+            ((__typeof__ (in[0])*)in)[k] = __bswap_32(((__typeof__ (in[0])*)in)[k]); \
+            break; \
+        case 4: \
+            for(k = 0; k < len; k++) \
+                ((__typeof__ (in[0])*)in)[k] = __bswap_64(((__typeof__ (in[0])*)in)[k]); \
+            break; \
+        } \
+    })
+#endif
+
 /**
 * \brief Fill the output buffer with the values of the
 * elements of the input stream by casting them to the
@@ -661,6 +797,27 @@ DLL_EXPORT void dsp_buffer_deviate(dsp_stream_p stream, double* deviation, doubl
     })
 #endif
 
+/**
+* \brief Fill the output buffer with the values of the
+* elements of the input stream by casting them to the
+* output buffer element type
+* \param in the input stream.
+* \param out the output stream.
+* \param len the length of the first input stream.
+* \param instep copy each instep elements of in into each outstep elements of out.
+* \param outstep copy each instep elements of in into each outstep elements of out.
+*/
+#ifndef dsp_buffer_copy_stepping
+#define dsp_buffer_copy_stepping(in, out, inlen, outlen, instep, outstep) \
+    ({ \
+    int k; \
+    int t; \
+        for(k = 0, t = 0; k < inlen && t < outlen; k+=instep, t+=outstep) { \
+        ((__typeof__ (out[0])*)out)[t] = (__typeof__ (out[0]))((__typeof__ (in[0])*)in)[k]; \
+        } \
+    })
+#endif
+
 /*@}*/
 /**
  * \defgroup dsp_DSPStream DSP API Stream type management functions
@@ -722,7 +879,7 @@ DLL_EXPORT dsp_stream_p dsp_stream_copy(dsp_stream_p stream);
 * \param stream the target DSP stream.
 * \param star the star to add to the stream.
 */
-DLL_EXPORT void dsp_stream_add_star(dsp_stream_p stream, dsp_star *star);
+DLL_EXPORT void dsp_stream_add_star(dsp_stream_p stream, dsp_star star);
 
 /**
 * \brief Add a child to the DSP Stream passed as argument
@@ -733,15 +890,33 @@ DLL_EXPORT void dsp_stream_add_star(dsp_stream_p stream, dsp_star *star);
 */
 DLL_EXPORT void dsp_stream_add_child(dsp_stream_p stream, dsp_stream_p child);
 
+/**
+* \brief Add a star to the DSP Stream passed as argument
+* \param stream the target DSP stream.
+* \param child the star to add to DSP stream.
+* \sa dsp_stream_new
+* \sa dsp_stream_del_star
+*/
+DLL_EXPORT void dsp_stream_add_star(dsp_stream_p stream, dsp_star star);
+
 /**
 * \brief Remove the child with index n to a DSP stream
 * \param stream the target DSP stream.
-* \param n the index of the dimension to remove
+* \param n the index of the child to remove
 * \sa dsp_stream_new
 * \sa dsp_stream_add_child
 */
 DLL_EXPORT void dsp_stream_del_child(dsp_stream_p stream, int n);
 
+/**
+* \brief Remove the star with index n to a DSP stream
+* \param stream the target DSP stream.
+* \param n the index of the star to remove
+* \sa dsp_stream_new
+* \sa dsp_stream_add_star
+*/
+DLL_EXPORT void dsp_stream_del_star(dsp_stream_p stream, int n);
+
 /**
 * \brief Add a dimension with length len to a DSP stream
 * \param stream the target DSP stream.
@@ -792,7 +967,7 @@ DLL_EXPORT int* dsp_stream_get_position(dsp_stream_p stream, int index);
 * \sa dsp_stream_get_position
 * \sa dsp_stream_set_position
 */
-DLL_EXPORT void *dsp_stream_exec(dsp_stream_p stream);
+DLL_EXPORT void *dsp_stream_exec(dsp_stream_p stream, void *args, ...);
 
 /**
 * \brief Crop the buffers of the stream passed as argument by reading the ROI field.
@@ -800,7 +975,7 @@ DLL_EXPORT void *dsp_stream_exec(dsp_stream_p stream);
 * \return the cropped DSP stream.
 * \sa dsp_stream_new
 */
-DLL_EXPORT dsp_stream_p dsp_stream_crop(dsp_stream_p stream);
+DLL_EXPORT void dsp_stream_crop(dsp_stream_p stream);
 
 /*@}*/
 /**
@@ -855,6 +1030,102 @@ DLL_EXPORT void dsp_modulation_frequency(dsp_stream_p stream, double samplefreq,
 */
 DLL_EXPORT void dsp_modulation_amplitude(dsp_stream_p stream, double samplefreq, double freq);
 
+/**
+* \brief Find stars into the stream
+* \param stream The stream containing stars
+* \param levels The level of thresholding
+* \param min_size Minimum stellar size
+* \param threshold Intensity treshold
+* \param matrix The star shape
+* \return The new dsp_stream_p structure pointer
+*/
+DLL_EXPORT int dsp_align_find_stars(dsp_stream_p stream, int levels, int min_size, float threshold, dsp_stream_p matrix);
+
+/**
+* \brief Limit search area to the radius around the first n stars and store those streams as children of the stream to be aligned
+* \param reference The reference solved stream
+* \param to_align The stream to be aligned
+* \param n Stars count limit
+* \param radius The search area
+* \return The number of streams cropped and added
+*/
+DLL_EXPORT int dsp_align_crop_limit(dsp_stream_p reference, dsp_stream_p to_align, int n, int radius);
+
+/**
+* \brief Find offsets between 2 streams and extract align informations
+* \param stream1 The first stream
+* \param stream2 The second stream
+* \param max_stars The maximum stars count allowed
+* \param precision The precision used for comparison
+* \param start_star Start compare from the start_star brigher star
+* \return The new dsp_align_info structure pointer
+*/
+DLL_EXPORT int dsp_align_get_offset(dsp_stream_p stream1, dsp_stream_p stream, int max_stars, int decimals);
+
+/**
+* \brief Rotate a stream around an axis and offset
+* \param stream The stream that need rotation
+* \param info The dsp_align_info structure pointer containing the rotation informations
+* \return The new dsp_stream_p structure pointer
+*/
+DLL_EXPORT void dsp_stream_rotate(dsp_stream_p stream);
+
+/**
+* \brief Traslate a stream
+* \param stream The stream that need traslation
+* \param info The dsp_align_info structure pointer containing the traslation informations
+* \return The new dsp_stream_p structure pointer
+*/
+DLL_EXPORT void dsp_stream_traslate(dsp_stream_p stream);
+
+/**
+* \brief Scale a stream
+* \param stream The stream that need scaling
+* \param info The dsp_align_info structure pointer containing the scaling informations
+* \return The new dsp_stream_p structure pointer
+*/
+DLL_EXPORT void dsp_stream_scale(dsp_stream_p stream);
+
+/**
+* \brief Read a FITS file and fill a dsp_stream_p with its content
+* \param filename the file name.
+* \param stretch 1 if the buffer intensities have to be stretched
+* \return The new dsp_stream_p structure pointer
+*/
+DLL_EXPORT dsp_stream_p* dsp_file_read_fits(char *filename, int *channels, int stretch);
+
+/**
+* \brief Write the components dsp_stream_p array into a FITS file,
+* \param filename the file name.
+* \param components the number of streams in the array to be used as components 1 or 3.
+* \param bpp the bit depth of the output JPEG file [8,16,32,64,-32,-64].
+* \param stream the input stream to be saved
+*/
+DLL_EXPORT void* dsp_file_write_fits(int bpp, size_t* memsize, dsp_stream_p stream);
+
+/**
+* \brief Read a JPEG file and fill a array of dsp_stream_p with its content,
+* each color channel has its own stream in this array and an additional grayscale at end will be added
+* \param filename the file name.
+* \param channels this value will be updated with the channel quantity into the picture.
+* \param stretch 1 if the buffer intensities have to be stretched
+* \return The new dsp_stream_p structure pointers array
+*/
+DLL_EXPORT dsp_stream_p* dsp_file_read_jpeg(char *filename, int *channels, int stretch);
+
+/**
+* \brief Write the components dsp_stream_p array into a JPEG file,
+* \param filename the file name.
+* \param components the number of streams in the array to be used as components 1 or 3.
+* \param quality the quality of the output JPEG file 0-100.
+* \param stream the input stream to be saved
+*/
+DLL_EXPORT void dsp_file_write_jpeg_composite(char *filename, int components, int quality, dsp_stream_p* stream);
+
+DLL_EXPORT dsp_stream_p *dsp_stream_from_components(dsp_t* buf, int dims, int *sizes, int components);
+DLL_EXPORT dsp_stream_p *dsp_buffer_rgb_to_components(void* buf, int dims, int *sizes, int components, int bpp, int stretch);
+DLL_EXPORT void dsp_buffer_components_to_rgb(dsp_stream_p *stream, void* rgb, int components, int bpp);
+
 /*@}*/
 /*@}*/
 
